@@ -15,6 +15,8 @@
 
 (** Pretty-printing.
 
+   If you are new to this module, see the {{!examples} examples} below.
+
    This module implements a pretty-printing facility to format values
    within {{!boxes}'pretty-printing boxes'} and {{!tags}'semantic tags'}
    combined with a set of {{!fpp}printf-like functions}.
@@ -41,9 +43,9 @@
    {!formatter_of_buffer}, {!formatter_of_symbolic_output_buffer} or using
    {{!section:formatter}custom formatters}.
 
-   Warning: Since {{!section:formatter}formatters} contain mutable state, it is
-   not thread-safe to use the same formatter on multiple domains in parallel
-   without synchronization.
+   {b Warning}: Since {{!section:formatter}formatters} contain
+   mutable state, it is not thread-safe to use the same formatter on multiple
+   domains in parallel without synchronization.
 
    If multiple domains write to the same output channel using the
    predefined formatters (as obtained by {!get_std_formatter} or
@@ -236,7 +238,7 @@ val print_string : string -> unit
 val pp_print_bytes : formatter -> bytes -> unit
 val print_bytes : bytes -> unit
 (** [pp_print_bytes ppf b] prints [b] in the current pretty-printing box.
-    @since 4.13.0
+    @since 4.13
 *)
 
 val pp_print_as : formatter -> int -> string -> unit
@@ -261,6 +263,11 @@ val pp_print_bool : formatter -> bool -> unit
 val print_bool : bool -> unit
 (** Print a boolean in the current pretty-printing box. *)
 
+val pp_print_nothing : formatter -> unit -> unit
+(** Print nothing.
+    @since 5.2
+*)
+
 (** {1:breaks Break hints} *)
 
 (** A 'break hint' tells the pretty-printer to output some space or split the
@@ -349,7 +356,7 @@ printf "@[<v 0>[@;<0 2>@[<v 0>a;@,b;@,c@]%t]@]@\n"
   (pp_print_custom_break ~fits:("", 0, "") ~breaks:(";", 0, ""))
    ]}
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val pp_force_newline : formatter -> unit -> unit
@@ -405,7 +412,8 @@ val print_newline : unit -> unit
 
   All open pretty-printing boxes are closed, all pending text is printed.
 
-  Equivalent to {!print_flush} followed by a new line.
+  Equivalent to {!print_flush} with a new line emitted on the pretty-printer
+  low-level output device immediately before the device is flushed.
   See corresponding words of caution for {!print_flush}.
 
   Note: this is not the normal way to output a new line;
@@ -415,6 +423,13 @@ val print_newline : unit -> unit
 
 (** {1 Margin} *)
 
+val pp_infinity : int
+(** [pp_infinity] is the maximal size of the margin.
+  Its exact value is implementation dependent but is guaranteed to be greater
+  than 10{^9}.
+
+  @since 5.2*)
+
 val pp_set_margin : formatter -> int -> unit
 val set_margin : int -> unit
 (** [pp_set_margin ppf d] sets the right margin to [d] (in characters):
@@ -423,8 +438,7 @@ val set_margin : int -> unit
   Setting the margin to [d] means that the formatting engine aims at
   printing at most [d-1] characters per line.
   Nothing happens if [d] is smaller than 2.
-  If [d] is too large, the right margin is set to the maximum
-  admissible value (which is greater than [10 ^ 9]).
+  If [d >= ]{!pp_infinity}, the right margin is set to {!pp_infinity}[ - 1].
   If [d] is less than the current maximum indentation limit, the
   maximum indentation limit is decreased while trying to preserve
   a minimal ratio [max_indent/margin>=50%] and if possible
@@ -470,11 +484,8 @@ val set_max_indent : int -> unit
 
   Nothing happens if [d] is smaller than 2.
 
-  If [d] is too large, the limit is set to the maximum
-  admissible value (which is greater than [10 ^ 9]).
-
-  If [d] is greater or equal than the current margin, it is ignored,
-  and the current maximum indentation limit is kept.
+  If [d] is greater than the current margin, it is ignored, and the current
+    maximum indentation limit is kept.
 
   See also {!pp_set_geometry}.
 *)
@@ -486,7 +497,7 @@ val get_max_indent : unit -> int
 (** {1 Geometry }
 
 Geometric functions can be used to manipulate simultaneously the
-coupled variables, margin and maxixum indentation limit.
+coupled variables, margin and maximum indentation limit.
 
 *)
 
@@ -494,8 +505,10 @@ type geometry = { max_indent:int; margin: int}
 (** @since 4.08 *)
 
 val check_geometry: geometry -> bool
-(** Check if the formatter geometry is valid: [1 < max_indent < margin]
-    @since 4.08 *)
+(** Check if the formatter geometry is valid:
+  [1 < max_indent < margin < ]{!pp_infinity}
+
+  @since 4.08 *)
 
 val pp_set_geometry : formatter -> max_indent:int -> margin:int -> unit
 val set_geometry : max_indent:int -> margin:int -> unit
@@ -505,7 +518,7 @@ val safe_set_geometry : max_indent:int -> margin:int -> unit
    [pp_set_geometry ppf ~max_indent ~margin] sets both the margin
    and maximum indentation limit for [ppf].
 
-   When [1 < max_indent < margin],
+   When [1 < max_indent < margin < ]{!pp_infinity},
    [pp_set_geometry ppf ~max_indent ~margin]
    is equivalent to
    [pp_set_margin ppf margin; pp_set_max_indent ppf max_indent];
@@ -515,7 +528,7 @@ val safe_set_geometry : max_indent:int -> margin:int -> unit
    Outside of this domain, [pp_set_geometry] raises an invalid argument
    exception whereas [pp_safe_set_geometry] does nothing.
 
-   @since 4.08.0
+   @since 4.08
 *)
 
 (**
@@ -526,7 +539,7 @@ val safe_set_geometry : max_indent:int -> margin:int -> unit
    Raises an invalid argument exception if the returned geometry
    does not satisfy {!check_geometry}.
 
-   @since 4.11.0
+   @since 4.11
 *)
 val pp_update_geometry : formatter -> (geometry -> geometry) -> unit
 val update_geometry : (geometry -> geometry) -> unit
@@ -535,7 +548,7 @@ val pp_get_geometry: formatter -> unit -> geometry
 val get_geometry: unit -> geometry
 (** Return the current geometry of the formatter
 
-    @since 4.08.0
+    @since 4.08
 *)
 
 
@@ -727,7 +740,7 @@ type stag += RGB of {r:int;g:int;b:int}
   Tag-marking operations may be set on or off with {!set_mark_tags}.
   Tag-printing operations may be set on or off with {!set_print_tags}.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 type tag = string
@@ -736,7 +749,7 @@ type stag += String_tag of tag
     by explicitly using the constructor [String_tag] or by using the dedicated
     format syntax ["@{<s> ... @}"].
 
-    @since 4.08.0
+    @since 4.08
 *)
 
 val pp_open_stag : formatter -> stag -> unit
@@ -747,7 +760,7 @@ val open_stag : stag -> unit
   [t] as argument; then the opening tag marker for [t], as given by
   [mark_open_stag t], is written into the output device of the formatter.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val pp_close_stag : formatter -> unit -> unit
@@ -758,7 +771,7 @@ val close_stag : unit -> unit
   output device of the formatter; then the [print_close_stag] tag-printing
   function of the formatter is called with [t] as argument.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val pp_set_tags : formatter -> bool -> unit
@@ -834,7 +847,7 @@ type formatter_out_functions = {
   out_flush : unit -> unit;
   out_newline : unit -> unit;
   out_spaces : int -> unit;
-  out_indent : int -> unit;(** @since 4.06.0 *)
+  out_indent : int -> unit;(** @since 4.06 *)
 }
 (** The set of output functions specific to a formatter:
 - the [out_string] function performs all the pretty-printer string output.
@@ -858,7 +871,7 @@ type formatter_out_functions = {
 - field [out_newline] is equivalent to [out_string "\n" 0 1];
 - fields [out_spaces] and [out_indent] are equivalent to
   [out_string (String.make n ' ') 0 n].
-  @since 4.01.0
+  @since 4.01
 *)
 
 val pp_set_formatter_out_functions :
@@ -876,7 +889,7 @@ val set_formatter_out_functions : formatter_out_functions -> unit
   Reasonable defaults for functions [out_spaces] and [out_newline] are
   respectively [out_funs.out_string (String.make n ' ') 0 n] and
   [out_funs.out_string "\n" 0 1].
-  @since 4.01.0
+  @since 4.01
 *)
 
 val pp_get_formatter_out_functions :
@@ -885,7 +898,7 @@ val get_formatter_out_functions : unit -> formatter_out_functions
 (** Return the current output functions of the pretty-printer,
   including line splitting and indentation functions. Useful to record the
   current setting and restore it afterwards.
-  @since 4.01.0
+  @since 4.01
 *)
 
 (** {1:tagsmeaning Redefining semantic tag operations} *)
@@ -903,7 +916,7 @@ type formatter_stag_functions = {
   [print] versions are the 'tag-printing' functions that can perform
   regular printing when a tag is closed or opened.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val pp_set_formatter_stag_functions :
@@ -925,7 +938,7 @@ val set_formatter_stag_functions : formatter_stag_functions -> unit
   are called at tag opening and tag closing time, to output regular material
   in the pretty-printer queue.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val pp_get_formatter_stag_functions :
@@ -934,7 +947,7 @@ val get_formatter_stag_functions : unit -> formatter_stag_functions
 (** Return the current semantic tag operation functions of the standard
     pretty-printer.
 
-    @since 4.08.0 *)
+    @since 4.08 *)
 
 (** {1:formatter Defining formatters}
 
@@ -1038,7 +1051,7 @@ val make_formatter :
   For instance,
   {[
     make_formatter
-      (Stdlib.output oc)
+      (Stdlib.output_substring oc)
       (fun () -> Stdlib.flush oc)
   ]}
   returns a formatter to the {!Stdlib.out_channel} [oc].
@@ -1065,7 +1078,7 @@ val formatter_of_out_functions :
   See definition of type {!formatter_out_functions} for the meaning of argument
   [out_funs].
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 
@@ -1102,34 +1115,34 @@ type symbolic_output_item =
   | Output_indent of int
   (** [Output_indent i]: symbolic indentation of size [i] *)
 (** Items produced by symbolic pretty-printers
-    @since 4.06.0
+    @since 4.06
 *)
 
 type symbolic_output_buffer
 (**
   The output buffer of a symbolic pretty-printer.
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val make_symbolic_output_buffer : unit -> symbolic_output_buffer
 (** [make_symbolic_output_buffer ()] returns a fresh buffer for
   symbolic output.
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val clear_symbolic_output_buffer : symbolic_output_buffer -> unit
 (** [clear_symbolic_output_buffer sob] resets buffer [sob].
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val get_symbolic_output_buffer :
   symbolic_output_buffer -> symbolic_output_item list
 (** [get_symbolic_output_buffer sob] returns the contents of buffer [sob].
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val flush_symbolic_output_buffer :
@@ -1140,34 +1153,60 @@ val flush_symbolic_output_buffer :
   [let items = get_symbolic_output_buffer sob in
    clear_symbolic_output_buffer sob; items]
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val add_symbolic_output_item :
   symbolic_output_buffer -> symbolic_output_item -> unit
 (** [add_symbolic_output_item sob itm] adds item [itm] to buffer [sob].
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 val formatter_of_symbolic_output_buffer : symbolic_output_buffer -> formatter
 (** [formatter_of_symbolic_output_buffer sob] returns a symbolic formatter
   that outputs to [symbolic_output_buffer] [sob].
 
-  @since 4.06.0
+  @since 4.06
 *)
 
 (** {1 Convenience formatting functions.} *)
 
+val pp_print_iter :
+  ?pp_sep:(formatter -> unit -> unit) ->
+  (('a -> unit) -> 'b -> unit) ->
+  (formatter -> 'a -> unit) -> formatter -> 'b -> unit
+(** [pp_print_iter ~pp_sep iter pp_v ppf v] formats on [ppf] the iterations of
+  [iter] over a collection [v] of values using [pp_v]. Iterations are
+  separated by [pp_sep] (defaults to {!pp_print_cut}).
+
+  @since 5.1
+*)
+
 val pp_print_list:
   ?pp_sep:(formatter -> unit -> unit) ->
   (formatter -> 'a -> unit) -> (formatter -> 'a list -> unit)
 (** [pp_print_list ?pp_sep pp_v ppf l] prints items of list [l],
   using [pp_v] to print each item, and calling [pp_sep]
-  between items ([pp_sep] defaults to {!pp_print_cut}.
+  between items ([pp_sep] defaults to {!pp_print_cut}).
   Does nothing on empty lists.
 
-  @since 4.02.0
+  @since 4.02
+*)
+
+val pp_print_array:
+  ?pp_sep:(formatter -> unit -> unit) ->
+  (formatter -> 'a -> unit) -> (formatter -> 'a array -> unit)
+(** [pp_print_array ?pp_sep pp_v ppf a] prints items of array [a],
+  using [pp_v] to print each item, and calling [pp_sep]
+  between items ([pp_sep] defaults to {!pp_print_cut}).
+  Does nothing on empty arrays.
+
+  If [a] is mutated after [pp_print_array] is called, the printed values
+  may not be what is expected because [Format] can delay the printing.
+  This can be avoided by flushing [ppf].
+
+  @since 5.1
 *)
 
 val pp_print_seq:
@@ -1187,7 +1226,7 @@ val pp_print_text : formatter -> string -> unit
 (** [pp_print_text ppf s] prints [s] with spaces and newlines respectively
   printed using {!pp_print_space} and {!pp_force_newline}.
 
-  @since 4.02.0
+  @since 4.02
 *)
 
 val pp_print_option :
@@ -1266,7 +1305,7 @@ val fprintf : formatter -> ('a, formatter, unit) format -> 'a
     optionally specified with the following syntax:
     the [<] character, followed by an integer [nspaces] value,
     then an integer [offset], and a closing [>] character.
-    If no parameters are provided, the good break defaults to a
+    If no parameters are provided, the full break defaults to a
     'space' break hint.
   - [@.]: flush the pretty-printer and split the line, as with
     [print_newline ()].
@@ -1327,7 +1366,8 @@ val sprintf : ('a, unit, string) format -> 'a
 (** Same as [printf] above, but instead of printing on a formatter,
   returns a string containing the result of formatting the arguments.
   Note that the pretty-printer queue is flushed at the end of {e each
-  call} to [sprintf].
+  call} to [sprintf]. Note that if your format string contains a [%a],
+  you should use [asprintf].
 
   In case of multiple and related calls to [sprintf] to output
   material on a single string, you should consider using [fprintf]
@@ -1345,7 +1385,7 @@ val asprintf : ('a, formatter, unit, string) format4 -> 'a
   The type of [asprintf] is general enough to interact nicely with [%a]
   conversions.
 
-  @since 4.01.0
+  @since 4.01
 *)
 
 val dprintf :
@@ -1367,7 +1407,7 @@ val dprintf :
   Format.printf "@[<v>%t@]" t
 ]}
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 
@@ -1375,7 +1415,7 @@ val ifprintf : formatter -> ('a, formatter, unit) format -> 'a
 (** Same as [fprintf] above, but does not print anything.
   Useful to ignore some material when conditionally printing.
 
-  @since 3.10.0
+  @since 3.10
 *)
 
 (** Formatted Pretty-Printing with continuations. *)
@@ -1392,7 +1432,7 @@ val kdprintf :
 (** Same as {!dprintf} above, but instead of returning immediately,
   passes the suspended printer to its first argument at the end of printing.
 
-  @since 4.08.0
+  @since 4.08
 *)
 
 val ikfprintf :
@@ -1401,7 +1441,7 @@ val ikfprintf :
 (** Same as [kfprintf] above, but does not print anything.
   Useful to ignore some material when conditionally printing.
 
-  @since 3.12.0
+  @since 3.12
 *)
 
 val ksprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b
@@ -1414,3 +1454,142 @@ val kasprintf : (string -> 'a) -> ('b, formatter, unit, 'a) format4 -> 'b
 
   @since 4.03
 *)
+
+(** {1:examples Examples}
+
+  A few warmup examples to get an idea of how Format is used.
+
+  We have a list [l] of pairs [(int * bool)], which the toplevel prints for us:
+
+  {[# let l = List.init 20 (fun n -> n, n mod 2 = 0)
+  val l : (int * bool) list =
+  [(0, true); (1, false); (2, true); (3, false); (4, true); (5, false);
+   (6, true); (7, false); (8, true); (9, false); (10, true); (11, false);
+   (12, true); (13, false); (14, true); (15, false); (16, true); (17, false);
+   (18, true); (19, false)]
+ ]}
+
+  If we want to print it ourself without the toplevel magic, we can try this:
+
+  {[
+  # let pp_pair out (x,y) = Format.fprintf out "(%d, %b)" x y
+  val pp_pair : Format.formatter -> int * bool -> unit = <fun>
+  # Format.printf "l: [@[<hov>%a@]]@."
+    Format.(pp_print_list ~pp_sep:(fun out () -> fprintf out ";@ ") pp_pair) l
+    l: [(0, true); (1, false); (2, true); (3, false); (4, true); (5, false);
+        (6, true); (7, false); (8, true); (9, false); (10, true); (11, false);
+        (12, true); (13, false); (14, true); (15, false); (16, true);
+        (17, false); (18, true); (19, false)]
+
+  ]}
+
+
+  What this does, briefly, is:
+
+    - [pp_pair] prints a pair [bool*int] surrounded in "(" ")". It takes
+      a formatter (into which formatting happens), and the pair itself.
+      When printing is done it returns [()].
+
+    - [Format.printf "l = [@[<hov>%a@]]@." ... l] is like [printf], but
+      with additional formatting instructions (denoted with "@"). The pair
+      "[@\[<hov>]" and "[@\]]" is a "horizontal-or-vertical box".
+
+    - "@." ends formatting with a newline. It is similar to "\n" but is also
+      aware of the [Format.formatter]'s state. Do not use "\n" with [Format].
+
+    - "%a" is a formatting instruction, like "%d" or "%s" for [printf].
+      However, where "%d" prints an integer and "%s" prints a string,
+      "%a" takes a printer (of type [Format.formatter -> 'a -> unit])
+      and a value (of type ['a]) and applies the printer to the value.
+      This is key to compositionality of printers.
+
+    - We build a list printer using
+      [Format.pp_print_list ~pp_sep:(...) pp_pair].
+      [pp_print_list] takes an element printer and returns a list printer.
+      The [?pp_sep] optional argument, if provided, is called in between
+      each element to print a separator.
+
+    - Here, for a separator, we use [(fun out () -> Format.fprintf out ";@ ")].
+      It prints ";", and then "@ " which is a breaking space
+      (either it prints " ", or it prints a newline if the box is about to
+      overflow).
+      This "@ " is responsible for the list printing splitting into several
+      lines.
+
+  If we omit "@ ", we get an ugly single-line print:
+
+  {[# Format.printf "l: [@[<hov>%a@]]@."
+      Format.(pp_print_list ~pp_sep:(fun out () -> fprintf out "; ") pp_pair) l
+  l: [(0, true); (1, false); (2, true); (* ... *); (18, true); (19, false)]
+- : unit = ()
+    ]}
+
+  Generally, it is good practice to define custom printers for important types
+  in your program. If, for example, you were to define basic geometry
+  types like so:
+
+  {[
+  type point = {
+    x: float;
+    y: float;
+  }
+
+  type rectangle = {
+    ll: point; (* lower left *)
+    ur: point; (* upper right *)
+  }
+  ]}
+
+  For debugging purpose, or to display information in logs, or on the console,
+  it would be convenient to define printers for these types.
+  Here is an example of to do it.
+  Note that "%.3f" is a [float] printer up to 3 digits of precision
+  after the dot; "%f" would print as many digits as required, which is
+  somewhat verbose; "%h" is an hexadecimal float printer.
+
+  {[
+  let pp_point out (p:point) =
+    Format.fprintf out "{ @[x=%.3f;@ y=%.3f@] }" p.x p.y
+
+  let pp_rectangle out (r:rectangle) =
+    Format.fprintf out "{ @[ll=%a;@ ur=%a@] }"
+      pp_point r.ll pp_point r.ur
+  ]}
+
+  In the [.mli] file, we could have:
+
+  {[
+    val pp_point : Format.formatter -> point -> unit
+
+    val pp_rectangle : Format.formatter -> rectangle -> unit
+  ]}
+
+  These printers can now be used with "%a" inside other printers.
+
+  {[ # Format.printf "some rectangle: %a@."
+        (Format.pp_print_option pp_rectangle)
+        (Some {ll={x=1.; y=2.}; ur={x=42.; y=500.12345}})
+  some rectangle: { l={ x=1.000; y=2.000 }; ur={ x=42.000; y=500.123 } }
+
+  # Format.printf "no rectangle: %a@."
+        (Format.pp_option pp_rectangle)
+        None
+  no rectangle:
+  ]}
+
+  See how we combine [pp_print_option] (option printer) and our newly defined
+  rectangle printer, like we did with [pp_print_list] earlier.
+
+  For a more extensive tutorial, see
+  {{: https://caml.inria.fr/resources/doc/guides/format.en.html}
+    "Using the Format module"}.
+
+  A final note: the [Format] module is a starting point.
+  The OCaml ecosystem has libraries that makes formatting easier
+  and more expressive, with more combinators, more concise names, etc.
+  An example of such a library is {{: https://erratique.ch/software/fmt} Fmt}.
+
+  Automatic deriving of pretty-printers from type definitions is also possible,
+  using {{: ppx_deriving.show} https://github.com/ocaml-ppx/ppx_deriving}
+  or similar ppx derivers.
+*)
